@@ -58,10 +58,17 @@ sub save {
       or Carp::croak "$CONFIG_FILE: $!";
 }
 
-sub node_config {
+sub config {
    \%CFG
 }
 
+sub find_profile($) {
+   +{
+      %CFG,
+      %{ $CFG{profile}{$_[0]} },
+   }
+}
+
 load;
 END { save }
 
